package com.hzya.frame.plugin.wscustomersynchronizationevent.service.impl;

import cn.hutool.core.map.MapBuilder;
import cn.hutool.crypto.SecureUtil;
import cn.hutool.http.HttpRequest;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.hzya.frame.basedao.service.impl.BaseService;
import com.hzya.frame.plugin.wscustomersynchronizationevent.dao.IWsCustomerSynchronizationEventDao;
import com.hzya.frame.plugin.wscustomersynchronizationevent.entity.WsCustomerSynchronizationEventEntity;
import com.hzya.frame.plugin.wscustomersynchronizationevent.service.IWsCustomerSynchronizationEventService;
import com.hzya.frame.web.entity.BaseResult;
import com.hzya.frame.web.entity.JsonResultEntity;
import com.sun.org.apache.xerces.internal.dom.DeferredEntityReferenceImpl;
import org.springframework.beans.factory.annotation.Autowired;

import java.util.Map;

/***
 * 万盛客商档案同步插件
 * @content:
 * @author 👻👻👻👻👻👻👻👻 gjh
 * @date 2025-03-03 10:56
 * @param
 * @return
 **/
public class WsCustomerSynchronizationEventServiceImpl extends BaseService<WsCustomerSynchronizationEventEntity, String> implements IWsCustomerSynchronizationEventService {
   //定义两个环境属性，根据环境绑定OA参数
    private final String TEST = "TEST";
    private final String DEV = "DEV";
    private IWsCustomerSynchronizationEventDao wsCustomerSynchronizationEventDao;

    @Autowired
    public void setWsCustomerSynchronizationEventDao(IWsCustomerSynchronizationEventDao wsCustomerSynchronizationEventDao) {
        this.wsCustomerSynchronizationEventDao = wsCustomerSynchronizationEventDao;
        this.dao = wsCustomerSynchronizationEventDao;
    }

    @Override
    public JsonResultEntity customerSynchronization(JSONObject requestJson) {
        JSONObject reqJson = new JSONObject();
        JSONObject jsonStr = requestJson.getJSONObject("jsonStr");
        JSONObject businessDataStr = jsonStr.getJSONObject("businessDataStr");
        String formAppId = requestJson.getString("formAppId");
        String eventType = requestJson.getString("eventType");
        switch (eventType) {
            case "onBeforeStart":
                //发起前事件，验证供应商重复
                return validateCustomerName(businessDataStr);
            case "onBeforeFinishWorkitem":
                //发起前事件，OA插件错误,临时配置事件
                return validateCustomerName(businessDataStr);
            case "onProcessFinished":
                //流程结束事件，同步客商到U8C
                return optionCustomerSynchronization(businessDataStr);
            default:
                return BaseResult.getFailureMessageEntity("未匹配到流程监听事件！当前传入的eventType：" + eventType);
        }

    }

    /****
     * 验证客商档案重复
     * @content:
     * @author 👻👻👻👻👻👻👻👻 gjh
     * @date 2025-03-03 15:22
     * @param
     * @return com.hzya.frame.web.entity.JsonResultEntity
     **/
    private JsonResultEntity validateCustomerName(JSONObject data) {
        JSONObject formBean = data.getJSONObject("formmain_4389");
        //客商名称
        String field0006 = formBean.getString("field0006").trim();
        //纳税人登记号
        String field0007 = formBean.getString("field0007");
        JSONObject requestJson = new JSONObject();
        requestJson.put("custnameis", field0006);
        String body = requestApi("validateCustomerName", requestJson, "800054", "8000540095", null);
        JSONObject req = JSON.parseObject(body);
        boolean flag = req.getBoolean("flag");
        if (flag) {
            JSONObject attribute = req.getJSONObject("attribute");
            String status = attribute.getString("status");
            if ("success".equals(status)) {
                JSONObject detailsData = attribute.getJSONObject("data");
                Integer allcount = detailsData.getInteger("allcount");
                if (allcount > 0) {
                    return BaseResult.getFailureMessageEntity("客商名称重复！");
                } else {
                    requestJson = new JSONObject();
                    if(null != field0007 && !"".equals(field0007)){
                        if(field0007.trim().length()>18){
                            logger.info("当前纳税人识别号接收数据field0007.trim():"+field0007.trim());
                            return BaseResult.getFailureMessageEntity("纳税人识别号不允许超过18位！");
                        }
                        requestJson.put("taxpayer", field0007.trim());
                        body = requestApi("validateCustomerName", requestJson, "800054", "8000540095", null);
                        req = JSON.parseObject(body);
                        flag = req.getBoolean("flag");
                        if (flag) {
                            attribute = req.getJSONObject("attribute");
                            status = attribute.getString("status");
                            if ("success".equals(status)) {
                                detailsData = attribute.getJSONObject("data");
                                allcount = detailsData.getInteger("allcount");
                                if (allcount > 0) {
                                    return BaseResult.getFailureMessageEntity("纳税人识别号重复！");
                                } else {
                                    return BaseResult.getSuccessMessageEntity("客商未重复！");
                                }
                            } else {
                                return BaseResult.getFailureMessageEntity("转发失败！");
                            }
                        } else {
                            return BaseResult.getFailureMessageEntity("转发失败！" + req.getString("msg"));
                        }
                    }else{
                        return BaseResult.getSuccessMessageEntity("纳税人识别号为空，不校验！");
                    }
                }
            } else {
                return BaseResult.getFailureMessageEntity("转发失败！");
            }
        } else {

            return BaseResult.getFailureMessageEntity("转发失败！" + req.getString("msg"));
        }


    }


    /****
     * 执行客商同步新增和分配
     * @content:
     * @author 👻👻👻👻👻👻👻👻 gjh
     * @date 2025-03-04 10:45
     * @param
     * @return com.hzya.frame.web.entity.JsonResultEntity
     **/
    private JsonResultEntity optionCustomerSynchronization(JSONObject data) {

        JSONObject formBean = data.getJSONObject("formmain_4389");
        //客商名称
        String field0006 = formBean.getString("field0006");
        //纳税人登记号
        String field0007 = formBean.getString("field0007");
        //绑定客商参数
        JSONObject requestJson = bindMerchantParameters(data, DEV);
        String apiCode = "8000540093";
        String body = requestApi("optionCustomerSynchronization客商新增", requestJson, "800054", apiCode, null);
        JSONObject req = JSON.parseObject(body);
        boolean flag = req.getBoolean("flag");
        String pk_cubasdoc = null;
        if (flag) {
            JSONObject attribute = req.getJSONObject("attribute");
            String status = attribute.getString("status");
            if ("success".equals(status)) {
                JSONArray returnData = attribute.getJSONArray("data");
                if (null != returnData && returnData.size() > 0) {
                    pk_cubasdoc = returnData.getJSONObject(0).getJSONObject("parentvo").getString("pk_cubasdoc");
                    String custcode = returnData.getJSONObject(0).getJSONObject("parentvo").getString("custcode");
                    //执行客商档案编码回传OA
                    updateCustomerCode(formBean.getString("id"), custcode,DEV);

                }
                logger.info("客商新增成功！返回数据", body);
            } else {
                logger.info("客商新增失败！返回数据", body);
            }
        } else {
            return BaseResult.getFailureMessageEntity("转发失败！" + req.getJSONObject("attribute").getString("errormsg"));
        }
        //绑定客商分配参数
        requestJson = bindMerchantDefineParameters(data, pk_cubasdoc,DEV);
        try {
            logger.info("新增客商后客商分配参数休眠10秒后触发");
            // 休眠 10 秒，1000 毫秒为 1 秒，10 秒则是 10 * 1000
            Thread.sleep(10 * 1000);
        } catch (InterruptedException e) {
            // 处理中断异常，这里简单打印异常信息
            logger.info("新增客商后客商分配参数休眠10秒失败"+e.getMessage());
            e.printStackTrace();
        }
        body = requestApi("optionCustomerSynchronization客商分配", requestJson, "800054", "8000540094", null);
        req = JSON.parseObject(body);
        flag = req.getBoolean("flag");
        if (flag) {
            JSONObject attribute = req.getJSONObject("attribute");
            String status = attribute.getString("status");
            if ("success".equals(status)) {
                logger.info("客商分配成功！返回数据", body);
            } else {
                logger.info("客商分配失败！返回数据", body);
            }
        } else {
            return BaseResult.getFailureMessageEntity("转发失败！" + req.getJSONObject("attribute").getString("errormsg"));
        }
        return BaseResult.getSuccessMessageEntity("执行客商同步方法成功");

    }

    /***
     * 更新OA客商编码
     * @content:
     * @author 👻👻👻👻👻👻👻👻 gjh
     * @date 2025-03-07 16:43
     * @param
     * @return com.alibaba.fastjson.JSONObject
     **/
    private JSONObject updateCustomerCode(String id, String custcode,String environment ) {
        String sql = "update formmain_4389 set field0005 = '" + custcode + "' where id =  " + id;


        String md5 = SecureUtil.md5(sql);
        System.out.println("明文摘要" + md5);
        String key = "`o7eotan9ws8c:aj";
        String encrypt = SecureUtil.aes(key.getBytes()).encryptHex(sql);
        JSONObject requestJson = new JSONObject();
        requestJson.put("userName", "hzya_rest");
        /** 生产环境和测试环境字段不一致，根据环境切换绑定*/
        String pk_currtype = null;
        if(DEV.equals(environment)){
            //所属地区编码
            requestJson.put("password", "23208c92-2e52-404a-a714-52061c2f915c");
//            requestJson.put("password", "2d6dd7fb-239a-4a0b-8c0a-5003647e9238");
        }else{
            requestJson.put("password", "23208c92-2e52-404a-a714-52061c2f915c");
        }
        requestJson.put("data", encrypt);
        requestJson.put("sign", md5);
        String tokenRest = requestApi("获取token", requestJson, "800055", "8000550010", null);
        String token = JSON.parseObject(tokenRest).getJSONObject("attribute").getString("id");
        String result = requestApi("更新A8客商档案", requestJson, "800055", "8000550000", token);
        //调用OArest接口
        logger.info("更新A8客商档案结果:" + result);

        return null;
    }

    /***
     * 绑定客商分配参数
     * @content:
     * @author 👻👻👻👻👻👻👻👻 gjh
     * @date 2025-03-05 15:02
     * @param
     * @param pk_cubasdoc
     * @return com.alibaba.fastjson.JSONObject
     **/
    private JSONObject bindMerchantDefineParameters(JSONObject data, String pk_cubasdoc,String environment) {
        JSONObject formBean = data.getJSONObject("formmain_4389");
        JSONObject returnData = new JSONObject();
        JSONArray custbasvo = new JSONArray();
        //客商档案
        JSONObject customer = new JSONObject();


        if(DEV.equals(environment)){
            //分配公司
            customer.put("pk_corp", formBean.getString("field0058"));
            //币种类别
            customer.put("pk_currtype1", formBean.getString("field0062"));
        }else{
            //分配公司
            customer.put("pk_corp", formBean.getString("field0064"));
            //币种类别
            customer.put("pk_currtype1", formBean.getString("field0057"));
        }


        String custprop = formBean.getString("field0027");
        //客商属性 0：客户，1：供应商，2 ：客商
        switch (custprop) {
            case "4109154014131407336":
                //客户
                custprop = "0";
                break;
            case "-6536959240623403744":
                //供应商
                custprop = "1";
                break;
            case "5695613751532077780":
                //客商
                custprop = "2";
                break;
            default:
                custprop = null;
                break;
        }
        customer.put("custprop", custprop);
        //客商ID
        customer.put("pk_cubasdoc", pk_cubasdoc);
        customer.put("creator", "admin");
        custbasvo.add(customer);
        returnData.put("custbasvo", custbasvo);
        return returnData;
    }

    /****
     * 请求API
     * @content:
     * @author 👻👻👻👻👻👻👻👻 gjh
     * @date 2025-03-04 16:52
     * @param
     * @return java.lang.String
     **/
    private String requestApi(String method, JSONObject requestJson, String appId, String apiCode, String token) {
        Map<String, String> headerMap = MapBuilder.<String, String>create(true)
                .put("apiCode", String.valueOf(apiCode))
                //这里用中台做为发送方
                .put("publicKey", "ZJYAWb7lhAUTYqekPkU+uHJv1/ObJxb7dT7sD8HPRDGAgyhCe7eDIk+3zDUT+v578prj")
                .put("secretKey", "fviZnLBsQUAGF8w8FSOdJi7XlIm/XAZclMxRagDLfTyJFlvnIBF3w66Hrpfzs8cYj3JzOP8MtA1LSGvL+2BWG8c/o7DKi92S4mr3zcGearA=")
                .put("appId", appId)
                .build();
        if (null != token) {
            headerMap.put("token", token);
        }
        String body = HttpRequest.post("http://127.0.0.1:10085/kangarooDataCenterV3/entranceController/externalCallInterface").addHeaders(headerMap).body(requestJson.toJSONString()).timeout(60000).execute().body();
        logger.info("中台转发到ESB" + method + ":{}", body);
        return body;
    }

    /****
     * 绑定客商新增OA和U8C客商参数
     * @content:
     * @author 👻👻👻👻👻👻👻👻 gjh
     * @date 2025-03-04 10:53
     * @param
     * @return com.alibaba.fastjson.JSONObject
     **/
    private JSONObject bindMerchantParameters(JSONObject data,String environment) {
        JSONObject formBean = data.getJSONObject("formmain_4389");
        JSONObject returnData = new JSONObject();
        JSONArray cbdocvo = new JSONArray();
        //客商档案
        JSONObject customer = new JSONObject();
        JSONArray banks = new JSONArray();
        JSONObject bank = new JSONObject();
        JSONObject parentvo = new JSONObject();
        bank.put("accaddr", "银行地址");

        /** 生产环境和测试环境字段不一致，根据环境切换绑定*/
        String pk_currtype = null;
        if(DEV.equals(environment)){
            //所属地区编码
            parentvo.put("pk_areacl", formBean.getString("field0060"));
            //币种类别
            pk_currtype = formBean.getString("field0062");
            //银行类别
            bank.put("banktypename", formBean.getString("field0064"));
        }else{
            //所属地区编码
            parentvo.put("pk_areacl", formBean.getString("field0062"));
            //币种类别
            pk_currtype = formBean.getString("field0057");
            //银行类别 TODO 修改为取编码
            bank.put("banktypename", formBean.getString("field0036"));
        }
        //账户属性
        String accountproperty = formBean.getString("field0042")== null?"":formBean.getString("field0042");
        switch (accountproperty) {
            case "-3164025096780690930":
                accountproperty = "电汇账户";
                break;
            case "7943667274710336919":
                accountproperty = "电汇和承兑账户";
                break;
            case "2952563644985590023":
                accountproperty = "承兑账户";
                break;
            default:
                accountproperty = null;
                break;
        }
        //账户性质 field0042
        bank.put("remcode", accountproperty);
        //银行账号
        bank.put("account", formBean.getString("field0011"));
        //账户名称
        bank.put("accountname", formBean.getString("field0012"));
        bank.put("defflag", "Y");
        bank.put("memo", formBean.getString("field0054"));
        //开户银行 传递以后名称
        bank.put("accname", formBean.getString("field0013"));
//        bank.put("pk_bankdoc", formBean.getString("field0013"));

        //新增固定传递集团
        bank.put("pk_corp", formBean.getString("0001"));

        bank.put("pk_currtype", pk_currtype);
        banks.add(bank);

        //创建人 U8C 制单人 admin
//        parentvo.put("creator",formBean.getString("field0019"));
        parentvo.put("creator", "admin");

        //客商名称
        parentvo.put("custname", formBean.getString("field0006").trim());

        //客商类型（0 - 外部单位1 - 内部核算单位2 - 内部法人单位3 - 内部渠道成员）
        String custprop = formBean.getString("field0026")== null?"":formBean.getString("field0026");
        switch (custprop) {
            case "-5116388890160378443":
                custprop = "0";
                break;
            case "8996290534573949076":
                custprop = "1";
                break;
            case "-2495523144656008721":
                custprop = "2";
                break;
            case "7042886455598794854":
                custprop = "3";
                break;
            default:
                custprop = null;
                break;
        }

        parentvo.put("custprop", custprop);
        //客商简称
        parentvo.put("custshortname", formBean.getString("field0001"));
        //销售方式
        String def1 = formBean.getString("field0024")== null?"":formBean.getString("field0024");

        switch (def1) {
            case "-285414580163689148":
                def1 = "经销";
                break;
            case "-3463202988521339445":
                def1 = "直销";
                break;
            default:
                def1 = null;
                break;
        }
        //销售方式
        parentvo.put("def1", def1);

        // 运输 自定义项8   001 是  002 否

        String def8 = formBean.getString("field0048")== null?"":formBean.getString("field0048");

        switch (def8) {
            case "-8661769604040273924":
                def8 = "001";
                break;
            case "-5350125814390239390":
                def8 = "002";
                break;
            default:
                def8 = null;
                break;
        }
        parentvo.put("def8", def8);
        //货代 自定义项7  001 是  002 否
        String def7 = formBean.getString("field0050") == null?"":formBean.getString("field0050");

        switch (def7) {
            case "-8661769604040273924":
                def7 = "001";
                break;
            case "-5350125814390239390":
                def7 = "002";
                break;
            default:
                def7 = null;
                break;
        }
        parentvo.put("def7", def7);

        //供应商类型 自定义项9  01 生产商  02 贸易商 03 服务商
        String def9 = formBean.getString("field0065") == null?"":formBean.getString("field0065");

        switch (def9) {

            case "4599320723300108944":
                def9 = "01";
                break;
            case "-2027733563790049244":
                def9 = "02";
                break;
            case "-2468198780444524154":
                def9 = "03";
                break;
            default:
                def9 = null;
                break;
        }

        parentvo.put("def9", def9 );

        //邮箱
        parentvo.put("email", formBean.getString("field0052"));
        //联系人1
        parentvo.put("linkman1", formBean.getString("field0010"));
        parentvo.put("memo", formBean.getString("field0054"));
        //通信地址
        parentvo.put("conaddr", formBean.getString("field0038"));
        //电话1  取 电话手机
        parentvo.put("phone1", formBean.getString("field0008"));

        //总公司名称 TODO 修改为取编码
        parentvo.put("pk_cubasdoc1", formBean.getString("field0034"));
        //纳税人识别号
        parentvo.put("taxpayerid", formBean.getString("field0007"));
        customer.put("parentvo", parentvo);
        customer.put("banks", banks);
        cbdocvo.add(customer);
        returnData.put("cbdocvo", cbdocvo);
        return returnData;
    }

}
